﻿using System;
using System.Collections.Generic;
using System.Text;

/**
 * Logger
 * 
 * @package psc-sdk
 * @copyright 2013 Paysafecard.com Wertkarten AG 
 * @version 1
 * @since 0.1
 */

namespace Logging
{
    /// <summary>
    /// Repräsentiert einen Standard-Logtyp
    /// </summary>
    public enum LogType
    {
        INFO = 0,
        WARNING,
        ERROR,
        DEBUG
    }

    public struct LogMessage
    {
        /// <summary>
        /// Repräsentiert eine Lognachricht
        /// </summary>
        /// <param name="message">Klartext-Lognachricht</param>
        /// <param name="call">Bezeichnung der Methode, die die Lognachricht generiert hat</param>
        /// <param name="call_params">Optionale Lognachrichten-Parameter</param>
        /// <param name="result">Beliebiger Resultcode</param>
        /// <param name="type">Beliebiger Wert von LogType</param>
        public LogMessage(string message, string call, string call_params, string result, LogType type = LogType.ERROR)
        {
            logType = type;
            this.message = message;
            callerFunc = call;
            callParams = call_params;
            requResult = result;
        }

        public LogType logType;
        public string message;
        public string callerFunc;
        public string callParams;
        public string requResult;
    }

    /// <summary>
    /// Logger-Klasse zur Speicherung von Lognachrichten und dessen Weiterleitung an den Anwendungscode
    /// </summary>
    public class Logger
    {
        public delegate void logMessageEventHandler(object sender, LogMessage logMessage);
        public event logMessageEventHandler logMessageEvent = null;

        private Dictionary<LogType, List<LogMessage>> m_logMessages;
        private bool m_logDebugMessages = false;

        public Logger()
        {
            m_logMessages = new Dictionary<LogType, List<LogMessage>>();

            m_logMessages.Add(LogType.DEBUG, new List<LogMessage>());
            m_logMessages.Add(LogType.WARNING, new List<LogMessage>());
            m_logMessages.Add(LogType.ERROR, new List<LogMessage>());
            m_logMessages.Add(LogType.INFO, new List<LogMessage>());
        }

        /// <summary>
        /// Gibt an, ob Debug-Lognachrichten ignoriert werden sollen
        /// </summary>
        public bool logDebugMessages
        {
            set { m_logDebugMessages = value; }
        }

        /// <summary>
        /// Fügt eine neue Lognachricht hinzu
        /// </summary>
        /// <param name="msg_code">Klartext-Lognachricht</param>
        /// <param name="call">Bezeichnung der Methode, die die Lognachricht generiert hat</param>
        /// <param name="call_params">Optionale Lognachrichten-Parameter</param>
        /// <param name="result">Beliebiger Resultcode</param>
        /// <param name="type">Beliebiger Wert von LogType</param>
        public void addLog(string msg_code, string call, string call_params, string result, LogType type = LogType.ERROR)
        {
            if (type == LogType.DEBUG)
            {
                // Debug-Logs sollen ignoriert werden, an dieser Stelle abbrechen:
                if (!m_logDebugMessages)
                    return;
            }

            LogMessage tmpLogMsg = new LogMessage(msg_code, call, call_params, result, type);
            m_logMessages[tmpLogMsg.logType].Add(tmpLogMsg);

            if (logMessageEvent != null)
                logMessageEvent(this, tmpLogMsg);
        }

        /// <summary>
        /// Fügt eine neue Debug-Lognachricht hinzu
        /// </summary>
        /// <param name="key">Beliebiger String, festgelegt durch die Applikation</param>
        /// <param name="value">Beliebiger String, festgelegt durch die Applikation</param>
        public void addDebug(string key, string value)
        {
            // Debug-Logs sollen ignoriert werden, an dieser Stelle abbrechen:
            if (!m_logDebugMessages)
                return;

            LogMessage tmpLogMsg = new LogMessage("0", key, null, value, Logging.LogType.DEBUG);
            m_logMessages[tmpLogMsg.logType].Add(tmpLogMsg);

            if (logMessageEvent != null)
                logMessageEvent(this, tmpLogMsg);
        }

        /// <summary>
        /// Gibt alle Lognachrichten des gewünschten Typs zurück
        /// </summary>
        /// <param name="logType"></param>
        /// <returns>Sammlung mit LogMessage-Objekten</returns>
        public List<LogMessage> getLogMessages(LogType logType = LogType.INFO)
        {
            List<LogMessage> logMsgs = new List<LogMessage>();

            foreach (LogMessage msg in m_logMessages[logType])
                logMsgs.Add(msg);

            return logMsgs;
        }

        /// <summary>
        /// Löscht alle Lognachrichten mit dem gewünschten Typ
        /// </summary>
        /// <param name="logType"></param>
        public void deleteLogMessages(LogType logType = LogType.INFO)
        {
            m_logMessages[logType].Clear();
        }
    }
}
